# Automatically generated code: EDIT AT YOUR OWN RISK
from traits import api as traits
from traitsui.item import Item, spring
from traitsui.group import HGroup
from traitsui.view import View

from tvtk import vtk_module as vtk
from tvtk import tvtk_base
from tvtk.tvtk_base_handler import TVTKBaseHandler
from tvtk import messenger
from tvtk.tvtk_base import deref_vtk
from tvtk import array_handler
from tvtk.array_handler import deref_array
from tvtk.tvtk_classes.tvtk_helper import wrap_vtk


def InstanceEditor(*args, **kw):
    from traitsui.editors.api import InstanceEditor as Editor
    return Editor(view_name="handler.view")

try:
    long
except NameError:
    # Silly workaround for Python3.
    long = int

from tvtk.tvtk_classes.object import Object


class PUnstructuredGridConnectivity(Object):
    """
    PUnstructuredGridConnectivity.h -- Unstructured grid connectivity.
    
    Superclass: Object
    
    PUnstructuredGridConnectivity implements functionality for
    generating
     ghost zones for a distributed unstructured grid. Generating ghost
    zones is
     implemented in two stages. First, we build the ghost zones, which
    amounts
     to building the connectivity of the ghosted grid and communication
    links,
     and second, we update the ghost zones by communicating the fields on
    the
     ghost cells and nodes. The main steps involved in this process are
    as
     follows:
     
       Each process computes a bounding box of the grid it owns. 
       The bounding boxes are then distributed to all processes by
            an all_gather collective call. 
       Each process loops through the list of bounding boxes and
            computes box intersections with its local bounding box. 
       The list of intersecting bounding boxes, yields an abbreviated
    list
            of candidate neighbors. 
       Given the local grid, each process then extracts the boundary
    grid,
            which consists of nodes/cells on the boundary, global node
    IDs and
            the local cell IDs w.r.t. the local grid. 
       Boundary grids are then exchanged among candidate neighbors using
            point-to-point communication. 
       Next, each process constructs the topology of the ghost zones and
            communication links, using the local boundary grid and the
    list of
            remote boundary grids. 
       The communication links store a source/target pair for nodes/cells
            among connected grids and remain persistent in memory. 
       Last, the fields (node- and/or cell-centered) are updated, using
            point-to-point communication by processing the communication
            links. 
     
    
    Caveats:
    
    
     
       The code currently assumes one grid per rank. 
       global_id information must be available. 
       The grid must be globally conforming, i.e., no hanging nodes. 
       Only topologically face-adjacent ghost cells are considered. 
       point_data and cell_data must match across partitions/processes. 
     
    
    See Also:
    
    
     PUnstructuredGridGhostDataGenerator
    
    """
    def __init__(self, obj=None, update=True, **traits):
        tvtk_base.TVTKBase.__init__(self, vtk.vtkPUnstructuredGridConnectivity, obj, update, **traits)
    
    def _get_controller(self):
        return wrap_vtk(self._vtk_obj.GetController())
    def _set_controller(self, arg):
        old_val = self._get_controller()
        self._wrap_call(self._vtk_obj.SetController,
                        deref_vtk(arg))
        self.trait_property_changed('controller', old_val, arg)
    controller = traits.Property(_get_controller, _set_controller, help=\
        """
        Set/Get the underlying MPI controller used for communication.
        """
    )

    global_id_field_name = traits.Trait(None, None, traits.String(enter_set=True, auto_set=False), help=\
        """
        Set/Get the name of the global_id field. By default, "_global_id" is
        assumed.
        """
    )

    def _global_id_field_name_changed(self, old_val, new_val):
        self._do_change(self._vtk_obj.SetGlobalIDFieldName,
                        self.global_id_field_name)

    def _get_ghosted_grid(self):
        return wrap_vtk(self._vtk_obj.GetGhostedGrid())
    ghosted_grid = traits.Property(_get_ghosted_grid, help=\
        """
        Returns the ghosted grid.
        """
    )

    def build_ghost_zone_connectivity(self):
        """
        V.build_ghost_zone_connectivity()
        C++: void BuildGhostZoneConnectivity()
        Builds the ghost-zone connectivity. This method sets up the
        necessary communication lists for updating the ghost zones. NOTE:
        the local grid, must be registered, by calling register_grid(),
        prior to calling this method.
        """
        ret = self._vtk_obj.BuildGhostZoneConnectivity()
        return ret
        

    def register_grid(self, *args):
        """
        V.register_grid(UnstructuredGrid)
        C++: void RegisterGrid(UnstructuredGrid *gridPtr)
        Registers the grid in this process
        """
        my_args = [deref_vtk(x) for x in args]
        ret = self._wrap_call(self._vtk_obj.RegisterGrid, *my_args)
        return ret

    def update_ghosts(self):
        """
        V.update_ghosts()
        C++: void UpdateGhosts()
        Exchanges ghost zone data (i.e., node-centered or cell-centered
        fields). NOTE: This method must be called after
        build_ghost_zone_connectivity.
        """
        ret = self._vtk_obj.UpdateGhosts()
        return ret
        

    _updateable_traits_ = \
    (('debug', 'GetDebug'), ('global_id_field_name',
    'GetGlobalIDFieldName'), ('reference_count', 'GetReferenceCount'),
    ('global_warning_display', 'GetGlobalWarningDisplay'))
    
    _allow_update_failure_ = \
    ()
    
    _full_traitnames_list_ = \
    (['debug', 'global_warning_display', 'global_id_field_name'])
    
    def trait_view(self, name=None, view_element=None):
        if view_element is not None or name not in (None, '', 'traits_view', 'full_traits_view', 'view'):
            return super(PUnstructuredGridConnectivity, self).trait_view(name, view_element)
        if name == 'full_traits_view':
            full_traits_view = \
            View((Item("handler._full_traits_list",show_label=False)),
            title='Edit PUnstructuredGridConnectivity properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return full_traits_view
        elif name == 'view':
            view = \
            View(([], [], ['global_id_field_name']),
            title='Edit PUnstructuredGridConnectivity properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return view
        elif name in (None, 'traits_view'):
            traits_view = \
            View((HGroup(spring, "handler.view_type", show_border=True), 
            Item("handler.info.object", editor = InstanceEditor(view_name="handler.view"), style = "custom", show_label=False)),
            title='Edit PUnstructuredGridConnectivity properties', scrollable=True, resizable=True,
            handler=TVTKBaseHandler,
            buttons=['OK', 'Cancel'])
            return traits_view
            

